@@ -120,6 +120,7 @@ stack::segment::segment(call *cr, int cid, int did, int tid) : OrderedObject()
 	sid.sequence = (uint32_t)now;
 	sid.sequence &= 0xffffffffl;
 	sid.expires = 0l;
+	sid.nat = NULL;
 	sid.cid = cid;
 	sid.did = did;
 	sid.tid = tid;
@@ -261,7 +262,6 @@ service::callback(1), mapped_array<MappedCall>(), OrderedIndex()
 	anon = "anonymous";
 	restricted = trusted = published = proxy = NULL;
 	localnames = "localhost, localhost.localdomain";
-	domain = NULL;
 	ring_timer = 4000;
 	cfna_timer = 16000;
 	reset_timer = 6000;
@@ -549,7 +549,6 @@ void stack::destroy(call *cr)
 	MappedCall *map;
 
 	linked_pointer<segment> sp;
-	rtpproxy *rtp = NULL;
 
 	cr->log();
 
@@ -577,17 +576,16 @@ void stack::destroy(call *cr)
 			}
 			sp->sid.delist(&hash[sp->sid.cid % keysize]);
 		}
+		if(sp->sid.nat)
+			media::release(&sp->sid.nat);
 		delete *sp;
 		sp = next;
 	}
-	rtp = cr->rtp;
 	map = cr->map;
 	cr->delist();
 	delete cr;
 	locking.share();
 	release(map);
-	if(rtp)
-		rtp->release();	
 }
 
 void stack::release(MappedCall *map)
@@ -794,7 +792,6 @@ void stack::reload(service *cfg)
 {
 	assert(cfg != NULL);	
 
-	const char *new_domain = "localdomain";
 	const char *new_proxy = NULL;
 	const char *key = NULL, *value;
 	linked_pointer<service::keynode> sp = cfg->getList("stack");
@@ -806,6 +803,8 @@ void stack::reload(service *cfg)
 	unsigned ring_value = 0;
 	unsigned reset_value = 0;
 
+	sip_domain = registry::getDomain();
+
 	while(sp) {
 		key = sp->getId();
 		value = sp->getPointer();
@@ -851,8 +850,6 @@ void stack::reload(service *cfg)
 				else
 					restricted = cfg->dup(value);
 			}
-			else if(!stricmp(key, "domain"))
-				new_domain = cfg->dup(value);
 			else if(!stricmp(key, "localnames"))
 				localhosts = cfg->dup(value);
 			else if(!stricmp(key, "trusted")) {
@@ -905,10 +902,9 @@ void stack::reload(service *cfg)
 
 	localnames = localhosts;
 	proxy = new_proxy;
-	domain = new_domain;
 
 	if(sip_family != AF_INET)
-		rtpproxy::enableIPV6();
+		media::enableIPV6();
 
 	if(ring_value && ring_value < 100)
 		ring_timer = ring_value * 1000l;
@@ -1012,7 +1008,7 @@ char *stack::sipAddress(struct sockaddr_internet *addr, char *buf, const char *u
 		break;
 #endif
 	default:
-		defaddr = stack::sip.domain;
+		defaddr = registry::getDomain();
 	}
 	if(!port)
 		port = sip_port;
@@ -1163,6 +1159,7 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	stack::call *call = s->parent;
 	linked_pointer<stack::segment> sp = call->segments.begin();
 	char username[MAX_USERID_SIZE];
+	char network[MAX_NETWORK_SIZE];
 	char touri[MAX_URI_SIZE];
 	char route[MAX_URI_SIZE];
 	osip_message_t *invite = NULL;
@@ -1171,9 +1168,30 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	int cid;
 	unsigned count = 0;
 	time_t now;
+	struct sockaddr_storage peering;
 
 	time(&now);
 
+	// compute network and subnet..
+	String::set(network, sizeof(network), "*");
+	uri::userid(uri_target, username, sizeof(username));
+	uri::hostid(uri_target, route, sizeof(route));
+
+	resolve.set(route, 5060);
+	target = resolve.getAddr();
+	if(target) {
+		stack::subnet *subnet = server::getPolicy(target);
+		if(subnet) {
+			memcpy(&peering, &subnet->iface, sizeof(struct sockaddr_storage));
+			String::set(network, sizeof(network), subnet->getId());
+		}
+		else
+			service::published(&peering);
+		server::release(subnet);
+	}
+	else
+		service::published(&peering);
+
 	// make sure we do not re-invite an existing active member again
 	while(is(sp)) {
 		if(!stricmp(sp->sid.identity, uri_target) && sp->sid.state != stack::session::CLOSED)
@@ -1198,7 +1216,7 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	osip_message_set_header(invite, ALLOW_EVENTS, "talk, hold, refer");
 	osip_message_set_supported(invite, "100rel,replaces,timer");
 
-	if(digest && s->reg) {
+	if(digest && s->reg) {	
 		char *authbuf = new char[1024];
 		stringbuf<64> response;
 		stringbuf<64> once;
@@ -1234,13 +1252,24 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 		delete[] authbuf;
 		osip_free(req); 
 	}
+	else
+		osip_message_set_header(invite, P_SIPWITCH_NODE, "no");
 
 	if(call->expires) {
 		snprintf(expheader, sizeof(expheader), "%ld", call->expires - now);
 		osip_message_set_header(invite, SESSION_EXPIRES, expheader);
 	}
 
-	osip_message_set_body(invite, s->sdp, strlen(s->sdp));
+	char sdp[MAX_SDP_BUFFER];
+	LinkedObject *nat = NULL;
+
+	if(media::invite(s, network, &nat, sdp) == NULL) {
+		process::errlog(ERRLOG, "cannot assign media proxy for %s", uri_target);
+		eXosip_unlock();
+		return;
+	}
+
+	osip_message_set_body(invite, sdp, strlen(sdp));
 	osip_message_set_content_type(invite, "application/sdp");
 	stack::siplog(invite);
 	cid = eXosip_call_send_initial_invite(invite);
@@ -1251,6 +1280,7 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 		++count;
 	}
 	else {
+		media::release(&nat);
 		process::errlog(ERRLOG, "invite failed for %s", uri_target);
 		eXosip_unlock();
 		return;
@@ -1259,23 +1289,14 @@ void stack::inviteRemote(stack::session *s, const char *uri_target, const char *
 	eXosip_unlock();
 	invited = stack::create(call, cid);
 	registry::incUse(NULL, stats::OUTGOING);
-	uri::userid(uri_target, username, sizeof(username));
-	uri::hostid(uri_target, route, sizeof(route));
 	String::set(invited->identity, sizeof(invited->identity), uri_target);
 	String::set(invited->display, sizeof(invited->display), username);
 	snprintf(invited->from, sizeof(invited->from), "<%s>", uri_target);
-	resolve.set(route, 5060);
-	service::published(&invited->peering);
-	String::set(invited->network, sizeof(invited->network), "*");
-	target = resolve.getAddr();
+	String::set(invited->network, sizeof(invited->network), network);
+	invited->nat = nat;
 	if(target) {
 		uri::identity(target, invited->sysident, username, sizeof(invited->sysident));
-		stack::subnet *subnet = server::getPolicy(target);
-		if(subnet) {
-			invited->peering = subnet->iface;
-			String::set(invited->network, sizeof(invited->network), subnet->getId());
-		}
-		server::release(subnet);
+		invited->peering = peering;
 	}
 	else
 		snprintf(invited->sysident, sizeof(invited->sysident), "%s@unknown", username);
@@ -1373,6 +1394,8 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 	stack::session *invited;
 	stack::call *call = s->parent;
 	linked_pointer<stack::segment> sp = call->segments.begin();
+	LinkedObject *nat;
+	char sdp[MAX_SDP_BUFFER];
 
 	time_t now;
 	osip_message_t *invite;
@@ -1409,13 +1432,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			goto next;
 		}
 
-		// if proxy required, but not available, then we must skip this
-		// invite...
-		// if(server::classify(&proxyinfo, &call->source->proxy, (struct sockaddr *)&tp->address) && !assign(call, 4))
-		//	goto next;
-
 		invite = NULL;
-		eXosip_lock();
 
 		if(dest == ROUTED) {
 			stack::sipPublish(&tp->address, route, call->dialed, sizeof(route));
@@ -1430,6 +1447,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 		route[0] = '<';
 		String::add(route, sizeof(route), ";lr>");
 
+		eXosip_lock();
 		if(eXosip_call_build_initial_invite(&invite, touri, s->from, route, call->subject)) {
 			stack::sipPublish(&tp->address, route, NULL, sizeof(route));
 			process::errlog(ERRLOG, "cannot invite %s; build failed", route);
@@ -1460,7 +1478,14 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			osip_message_set_header(invite, SESSION_EXPIRES, expheader);
 		}
 
-		osip_message_set_body(invite, s->sdp, strlen(s->sdp));
+		nat = NULL;
+		if(media::invite(s, tp->network, &nat, sdp) == NULL) {
+			stack::sipPublish(&tp->address, route, NULL, sizeof(route));
+			process::errlog(ERRLOG, "no media proxy available for %s", route);
+			goto unlock;
+		}
+
+		osip_message_set_body(invite, sdp, strlen(sdp));
 		osip_message_set_content_type(invite, "application/sdp");
 		stack::siplog(invite);
 		cid = eXosip_call_send_initial_invite(invite);
@@ -1471,6 +1496,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 			++count;
 		}
 		else {
+			media::release(&nat);
 			stack::sipPublish(&tp->address, route, NULL, sizeof(route));
 			process::errlog(ERRLOG, "invite failed for %s", route);
 			goto unlock;
@@ -1482,6 +1508,7 @@ void stack::inviteLocal(stack::session *s, registry::mapped *rr, destination_t d
 
 		String::set(invited->network, sizeof(invited->network), tp->network);
 		invited->peering = tp->peering;
+		invited->nat = nat;
 		
 		if(rr->ext) 
 			snprintf(invited->sysident, sizeof(invited->sysident), "%u", rr->ext);
@@ -1540,16 +1567,4 @@ next:
 	}
 }
 
-bool stack::assign(stack::call *cr, unsigned count)
-{
-	if(cr->rtp)
-		return true;
-
-	cr->rtp = rtpproxy::create(count);
-	if(cr->rtp)
-		return true;
-
-	return false;
-}
-
 END_NAMESPACE
